@@ -59,7 +59,8 @@ typedef struct _GeditPanelItem GeditPanelItem;
 
 struct _GeditPanelItem 
 {
-	gchar *name;
+	gchar *id;
+	gchar *display_name;
 	GtkWidget *icon;
 };
 
@@ -142,14 +143,14 @@ static void
 gedit_panel_focus_document (GeditPanel *panel)
 {
 	GtkWidget *toplevel = gtk_widget_get_toplevel (GTK_WIDGET (panel));
-	if (GTK_WIDGET_TOPLEVEL (toplevel) && GEDIT_IS_WINDOW (toplevel))
-   	{
+	if (gtk_widget_is_toplevel (toplevel) && GEDIT_IS_WINDOW (toplevel))
+	{
 		GeditView *view;
 
 		view = gedit_window_get_active_view (GEDIT_WINDOW (toplevel));
 		if (view != NULL)
 			gtk_widget_grab_focus (GTK_WIDGET (view));
-   	}
+	}
 }
 
 static void
@@ -264,74 +265,74 @@ set_gtk_image_from_gtk_image (GtkImage *image,
 {
 	switch (gtk_image_get_storage_type (source))
 	{
-	case GTK_IMAGE_EMPTY:
-		gtk_image_clear (image);
-		break;
-	case GTK_IMAGE_PIXMAP:
-		{
-			GdkPixmap *pm;
-			GdkBitmap *bm;
+		case GTK_IMAGE_EMPTY:
+			gtk_image_clear (image);
+			break;
+		case GTK_IMAGE_PIXMAP:
+			{
+				GdkPixmap *pm;
+				GdkBitmap *bm;
 
-			gtk_image_get_pixmap (source, &pm, &bm);
-			gtk_image_set_from_pixmap (image, pm, bm);
-		}
-		break;
-	case GTK_IMAGE_IMAGE:
-		{
-			GdkImage *i;
-			GdkBitmap *bm;
+				gtk_image_get_pixmap (source, &pm, &bm);
+				gtk_image_set_from_pixmap (image, pm, bm);
+			}
+			break;
+		case GTK_IMAGE_IMAGE:
+			{
+				GdkImage *i;
+				GdkBitmap *bm;
 
-			gtk_image_get_image (source, &i, &bm);
-			gtk_image_set_from_image (image, i, bm);
-		}
-		break;
-	case GTK_IMAGE_PIXBUF:
-		{
-			GdkPixbuf *pb;
+				gtk_image_get_image (source, &i, &bm);
+				gtk_image_set_from_image (image, i, bm);
+			}
+			break;
+		case GTK_IMAGE_PIXBUF:
+			{
+				GdkPixbuf *pb;
 
-			pb = gtk_image_get_pixbuf (source);
-			gtk_image_set_from_pixbuf (image, pb);
-		}
-		break;
-	case GTK_IMAGE_STOCK:
-		{
-			gchar *s_id;
-			GtkIconSize s;
+				pb = gtk_image_get_pixbuf (source);
+				gtk_image_set_from_pixbuf (image, pb);
+			}
+			break;
+		case GTK_IMAGE_STOCK:
+			{
+				gchar *s_id;
+				GtkIconSize s;
 
-			gtk_image_get_stock (source, &s_id, &s);
-			gtk_image_set_from_stock (image, s_id, s);
-		}
-		break;
-	case GTK_IMAGE_ICON_SET:
-		{
-			GtkIconSet *is;
-			GtkIconSize s;
+				gtk_image_get_stock (source, &s_id, &s);
+				gtk_image_set_from_stock (image, s_id, s);
+			}
+			break;
+		case GTK_IMAGE_ICON_SET:
+			{
+				GtkIconSet *is;
+				GtkIconSize s;
 
-			gtk_image_get_icon_set (source, &is, &s);
-			gtk_image_set_from_icon_set (image, is, s);
-		}
-		break;
-	case GTK_IMAGE_ANIMATION:
-		{
-			GdkPixbufAnimation *a;
+				gtk_image_get_icon_set (source, &is, &s);
+				gtk_image_set_from_icon_set (image, is, s);
+			}
+			break;
+		case GTK_IMAGE_ANIMATION:
+			{
+				GdkPixbufAnimation *a;
 
-			a = gtk_image_get_animation (source);
-			gtk_image_set_from_animation (image, a);
-		}
-		break;
-	case GTK_IMAGE_ICON_NAME:
-		{
-			const gchar *n;
-			GtkIconSize s;
+				a = gtk_image_get_animation (source);
+				gtk_image_set_from_animation (image, a);
+			}
+			break;
+		case GTK_IMAGE_ICON_NAME:
+			{
+				const gchar *n;
+				GtkIconSize s;
 
-			gtk_image_get_icon_name (source, &n, &s);
-			gtk_image_set_from_icon_name (image, n, s);
-		}
-		break;
-	default:
-		gtk_image_set_from_stock (image,
-					  GTK_STOCK_FILE,
-					  GTK_ICON_SIZE_MENU);
+				gtk_image_get_icon_name (source, &n, &s);
+				gtk_image_set_from_icon_name (image, n, s);
+			}
+			break;
+		default:
+			gtk_image_set_from_stock (image,
+						  GTK_STOCK_FILE,
+						  GTK_ICON_SIZE_MENU);
 	}
 }
 
@@ -345,7 +346,7 @@ sync_title (GeditPanel     *panel,
 	if (item != NULL)
 	{
 		gtk_label_set_text (GTK_LABEL (panel->priv->title_label), 
-				    item->name);
+				    item->display_name);
 
 		set_gtk_image_from_gtk_image (GTK_IMAGE (panel->priv->title_image),
 					      GTK_IMAGE (item->icon));
@@ -362,10 +363,10 @@ sync_title (GeditPanel     *panel,
 }
 
 static void
-notebook_page_changed (GtkNotebook     *notebook,
-                       GtkNotebookPage *page,
-                       guint            page_num,
-                       GeditPanel      *panel)
+notebook_page_changed (GtkNotebook *notebook,
+                       GtkWidget   *page,
+                       guint        page_num,
+                       GeditPanel  *panel)
 {
 	GtkWidget *item;
 	GeditPanelItem *data;
@@ -596,7 +597,9 @@ gedit_panel_constructor (GType type,
 GtkWidget *
 gedit_panel_new (GtkOrientation orientation)
 {
-	return GTK_WIDGET (g_object_new (GEDIT_TYPE_PANEL, "orientation", orientation, NULL));
+	return GTK_WIDGET (g_object_new (GEDIT_TYPE_PANEL,
+					 "orientation", orientation,
+					 NULL));
 }
 
 static GtkWidget *
@@ -641,19 +644,51 @@ build_tab_label (GeditPanel  *panel,
 	return hbox;
 }
 
+static gboolean
+item_exists (GeditPanel  *panel,
+	     const gchar *id)
+{
+	GeditPanelItem *data;
+	GList *items, *l;
+	gboolean exists = FALSE;
+
+	items = gtk_container_get_children (GTK_CONTAINER (panel->priv->notebook));
+
+	for (l = items; l != NULL; l = g_list_next (l))
+	{
+		data = (GeditPanelItem *)g_object_get_data (G_OBJECT (l->data),
+						            PANEL_ITEM_KEY);
+		g_return_val_if_fail (data != NULL, FALSE);
+
+		if (strcmp (data->id, id) == 0)
+		{
+			exists = TRUE;
+			break;
+		}
+	}
+
+	g_list_free (items);
+
+	return exists;
+}
+
 /**
  * gedit_panel_add_item:
  * @panel: a #GeditPanel
  * @item: the #GtkWidget to add to the @panel
- * @name: the name to be shown in the @panel
- * @image: the image to be shown in the @panel
+ * @id: unique name for the new item
+ * @display_name: the name to be shown in the @panel
+ * @image: (allow-none): the image to be shown in the @panel, or %NULL
  *
  * Adds a new item to the @panel.
+ *
+ * Returns: %TRUE is the item was successfully added.
  */
-void
-gedit_panel_add_item (GeditPanel  *panel, 
-		      GtkWidget   *item, 
-		      const gchar *name,
+gboolean
+gedit_panel_add_item (GeditPanel  *panel,
+		      GtkWidget   *item,
+		      const gchar *id,
+		      const gchar *display_name,
 		      GtkWidget   *image)
 {
 	GeditPanelItem *data;
@@ -661,14 +696,21 @@ gedit_panel_add_item (GeditPanel  *panel,
 	GtkWidget *menu_label;
 	gint w, h;
 	
-	g_return_if_fail (GEDIT_IS_PANEL (panel));
-	g_return_if_fail (GTK_IS_WIDGET (item));
-	g_return_if_fail (name != NULL);
-	g_return_if_fail (image == NULL || GTK_IS_IMAGE (image));
+	g_return_val_if_fail (GEDIT_IS_PANEL (panel), FALSE);
+	g_return_val_if_fail (GTK_IS_WIDGET (item), FALSE);
+	g_return_val_if_fail (id != NULL, FALSE);
+	g_return_val_if_fail (display_name != NULL, FALSE);
+	g_return_val_if_fail (image == NULL || GTK_IS_IMAGE (image), FALSE);
 
-	data = g_new (GeditPanelItem, 1);
+	if (item_exists (panel, id))
+	{
+		g_critical ("You are trying to add an item with an id that already exists");
+		return FALSE;
+	}
 
-	data->name = g_strdup (name);
+	data = g_slice_new (GeditPanelItem);
+	data->id = g_strdup (id);
+	data->display_name = g_strdup (display_name);
 
 	if (image == NULL)
 	{
@@ -688,12 +730,12 @@ gedit_panel_add_item (GeditPanel  *panel,
 		           PANEL_ITEM_KEY,
 		           data);
 
-	tab_label = build_tab_label (panel, item, data->name, data->icon);
+	tab_label = build_tab_label (panel, item, data->display_name, data->icon);
 
-	menu_label = gtk_label_new (name);
+	menu_label = gtk_label_new (display_name);
 	gtk_misc_set_alignment (GTK_MISC (menu_label), 0.0, 0.5);
 
-	if (!GTK_WIDGET_VISIBLE (item))
+	if (!gtk_widget_get_visible (item))
 		gtk_widget_show (item);
 
 	gtk_notebook_append_page_menu (GTK_NOTEBOOK (panel->priv->notebook),
@@ -702,21 +744,27 @@ gedit_panel_add_item (GeditPanel  *panel,
 				       menu_label);
 
 	g_signal_emit (G_OBJECT (panel), signals[ITEM_ADDED], 0, item);
+
+	return TRUE;
 }
 
 /**
  * gedit_panel_add_item_with_stock_icon:
  * @panel: a #GeditPanel
  * @item: the #GtkWidget to add to the @panel
- * @name: the name to be shown in the @panel
- * @stock_id: a stock id
+ * @id: unique name for the new item
+ * @display_name: the name to be shown in the @panel
+ * @stock_id: (allow-none): a stock id, or %NULL
  *
  * Same as gedit_panel_add_item() but using an image from stock.
+ *
+ * Returns: %TRUE is the item was successfully added.
  */
-void
-gedit_panel_add_item_with_stock_icon (GeditPanel  *panel, 
-				      GtkWidget   *item, 
-				      const gchar *name,
+gboolean
+gedit_panel_add_item_with_stock_icon (GeditPanel  *panel,
+				      GtkWidget   *item,
+				      const gchar *id,
+				      const gchar *display_name,
 				      const gchar *stock_id)
 {
 	GtkWidget *icon = NULL;
@@ -727,7 +775,7 @@ gedit_panel_add_item_with_stock_icon (GeditPanel  *panel,
 						 GTK_ICON_SIZE_MENU);
 	}
 
-	gedit_panel_add_item (panel, item, name, icon);
+	return gedit_panel_add_item (panel, item, id, display_name, icon);
 }
 
 /**
@@ -736,9 +784,9 @@ gedit_panel_add_item_with_stock_icon (GeditPanel  *panel,
  * @item: the item to be removed from the panel
  *
  * Removes the widget @item from the panel if it is in the @panel and returns
- * TRUE if there was not any problem.
+ * %TRUE if there was not any problem.
  *
- * Returns: TRUE if it was well removed.
+ * Returns: %TRUE if it was well removed.
  */
 gboolean
 gedit_panel_remove_item (GeditPanel *panel,
@@ -752,16 +800,17 @@ gedit_panel_remove_item (GeditPanel *panel,
 
 	page_num = gtk_notebook_page_num (GTK_NOTEBOOK (panel->priv->notebook),
 					  item);
-					  
+
 	if (page_num == -1)
 		return FALSE;
 		
 	data = (GeditPanelItem *)g_object_get_data (G_OBJECT (item),
 					            PANEL_ITEM_KEY);
 	g_return_val_if_fail (data != NULL, FALSE);
-	
-	g_free (data->name);
-	g_free (data);
+
+	g_free (data->id);
+	g_free (data->display_name);
+	g_slice_free (GeditPanelItem, data);
 
 	g_object_set_data (G_OBJECT (item),
 		           PANEL_ITEM_KEY,
@@ -791,7 +840,7 @@ gedit_panel_remove_item (GeditPanel *panel,
  *
  * Switches to the page that contains @item.
  *
- * Returns: TRUE if it was activated
+ * Returns: %TRUE if it was activated
  */
 gboolean
 gedit_panel_activate_item (GeditPanel *panel,
@@ -817,11 +866,11 @@ gedit_panel_activate_item (GeditPanel *panel,
 /**
  * gedit_panel_item_is_active:
  * @panel: a #GeditPanel
- * @item: a widget contained in #GeditPanel
+ * @item: a #GtkWidget
  *
- * Wheter @item is the one current active in @panel
+ * Returns whether @item is the active widget in @panel
  *
- * Returns: TRUE if the widget is active
+ * Returns: %TRUE if @item is the active widget
  */
 gboolean
 gedit_panel_item_is_active (GeditPanel *panel,
@@ -895,18 +944,11 @@ _gedit_panel_get_active_item_id (GeditPanel *panel)
 				GTK_NOTEBOOK (panel->priv->notebook),
 				cur_page);
 
-	/* FIXME: for now we use as the hash of the name as id.
-	 * However the name is not guaranteed to be unique and
-	 * it is a translated string, so it's subotimal, but should
-	 * be good enough for now since we don't want to add an
-	 * ad hoc id argument.
-	 */
-
 	data = (GeditPanelItem *)g_object_get_data (G_OBJECT (item),
 					            PANEL_ITEM_KEY);
 	g_return_val_if_fail (data != NULL, 0);
 
-	return g_str_hash (data->name);
+	return g_str_hash (data->id);
 }
 
 void
@@ -935,7 +977,7 @@ _gedit_panel_set_active_item_by_id (GeditPanel *panel,
 						            PANEL_ITEM_KEY);
 		g_return_if_fail (data != NULL);
 
-		if (g_str_hash (data->name) == id)
+		if (g_str_hash (data->id) == id)
 		{
 			gtk_notebook_set_current_page (
 				GTK_NOTEBOOK (panel->priv->notebook), i);
@@ -944,3 +986,5 @@ _gedit_panel_set_active_item_by_id (GeditPanel *panel,
 		}
 	}
 }
+
+/* ex:ts=8:noet: */
