@@ -129,6 +129,7 @@ static gboolean gedit_view_drag_drop		(GtkWidget        *widget,
 	      					 guint             timestamp);
 static gboolean	gedit_view_button_press_event	(GtkWidget        *widget,
 						 GdkEventButton   *event);
+static void	gedit_view_realize		(GtkWidget        *widget);
 
 static gboolean start_interactive_search	(GeditView        *view);
 static gboolean start_interactive_goto_line	(GeditView        *view);
@@ -214,6 +215,7 @@ gedit_view_class_init (GeditViewClass *klass)
 	widget_class->drag_data_received = gedit_view_drag_data_received;
 	widget_class->drag_drop = gedit_view_drag_drop;
 	widget_class->button_press_event = gedit_view_button_press_event;
+	widget_class->realize = gedit_view_realize;
 	klass->start_interactive_search = start_interactive_search;
 	klass->start_interactive_goto_line = start_interactive_goto_line;
 	klass->reset_searched_text = reset_searched_text;	
@@ -315,7 +317,7 @@ extension_added (PeasExtensionSet *extensions,
 		 PeasExtension    *exten,
 		 GeditView        *view)
 {
-	peas_extension_call (exten, "activate", view);
+	peas_extension_call (exten, "activate");
 }
 
 static void
@@ -324,7 +326,7 @@ extension_removed (PeasExtensionSet *extensions,
 		   PeasExtension    *exten,
 		   GeditView        *view)
 {
-	peas_extension_call (exten, "deactivate", view);
+	peas_extension_call (exten, "deactivate");
 }
 
 static void
@@ -353,11 +355,6 @@ on_notify_buffer_cb (GeditView  *view,
 			  "search_highlight_updated",
 			  G_CALLBACK (search_highlight_updated_cb),
 			  view);
-
-	/* We only activate the extensions when the right buffer is set,
-	 * because most plugins will expect this behaviour, and we won't
-	 * change the buffer later anyway. */
-	peas_extension_set_call (view->priv->extensions, "activate", view);
 }
 
 static void 
@@ -451,7 +448,9 @@ gedit_view_init (GeditView *view)
 		gtk_target_list_add_uri_targets (tl, TARGET_URI_LIST);
 		
 	view->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
-							 GEDIT_TYPE_VIEW_ACTIVATABLE);
+							 GEDIT_TYPE_VIEW_ACTIVATABLE,
+							 "view", view,
+							 NULL);
 	g_signal_connect (view->priv->extensions,
 			  "extension-added",
 			  G_CALLBACK (extension_added),
@@ -466,7 +465,6 @@ gedit_view_init (GeditView *view)
 			  "notify::buffer", 
 			  G_CALLBACK (on_notify_buffer_cb),
 			  NULL);
-
 }
 
 static void
@@ -476,10 +474,12 @@ gedit_view_destroy (GtkObject *object)
 
 	view = GEDIT_VIEW (object);
 
-	peas_extension_set_call (view->priv->extensions,
-				 "deactivate",
-				 view);
-	g_object_unref (view->priv->extensions);
+	if (view->priv->extensions != NULL)
+	{
+		peas_extension_set_call (view->priv->extensions, "deactivate");
+		g_object_unref (view->priv->extensions);
+		view->priv->extensions = NULL;
+	}
 
 	if (view->priv->search_window != NULL)
 	{
@@ -1285,24 +1285,11 @@ search_enable_popdown (GtkWidget *widget,
 }
 
 static void
-search_entry_populate_popup (GtkEntry  *entry,
-			     GtkMenu   *menu,
-			     GeditView *view)
+add_popup_menu_items (GtkWidget *menu,
+		      GeditView *view)
 {
 	GtkWidget *menu_item;
 
-	view->priv->disable_popdown = TRUE;
-	g_signal_connect (menu, "hide",
-		    	  G_CALLBACK (search_enable_popdown), view);
-
-	if (view->priv->search_mode == GOTO_LINE)
-		return;
-
-	/* separator */
-	menu_item = gtk_menu_item_new ();
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
-	gtk_widget_show (menu_item);
-
 	/* create "Wrap Around" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Wrap Around"));
 	g_signal_connect (G_OBJECT (menu_item), "toggled",
@@ -1334,6 +1321,55 @@ search_entry_populate_popup (GtkEntry  *entry,
 	gtk_widget_show (menu_item);
 }
 
+static void
+search_entry_populate_popup (GtkEntry  *entry,
+			     GtkMenu   *menu,
+			     GeditView *view)
+{
+	GtkWidget *menu_item;
+
+	view->priv->disable_popdown = TRUE;
+	g_signal_connect (menu, "hide",
+			  G_CALLBACK (search_enable_popdown), view);
+
+	if (view->priv->search_mode == GOTO_LINE)
+		return;
+
+	/* separator */
+	menu_item = gtk_menu_item_new ();
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
+	gtk_widget_show (menu_item);
+
+	add_popup_menu_items (GTK_WIDGET (menu), view);
+}
+
+static void
+search_entry_icon_release (GtkEntry            *entry,
+			   GtkEntryIconPosition icon_pos,
+			   GdkEventButton      *event,
+			   GeditView           *view)
+{
+	GtkWidget *menu;
+
+	if (view->priv->search_mode == GOTO_LINE ||
+	    icon_pos != GTK_ENTRY_ICON_PRIMARY)
+		return;
+
+	menu = gtk_menu_new ();
+	gtk_widget_show (menu);
+
+	view->priv->disable_popdown = TRUE;
+	g_signal_connect (menu, "hide",
+			  G_CALLBACK (search_enable_popdown), view);
+
+	add_popup_menu_items (menu, view);
+
+	gtk_menu_popup (GTK_MENU (menu),
+			NULL, NULL,
+			NULL, NULL,
+			event->button, event->time);
+}
+
 static void
 search_entry_insert_text (GtkEditable *editable, 
 			  const gchar *text, 
@@ -1575,10 +1611,13 @@ ensure_search_window (GeditView *view)
 	/* add entry */
 	view->priv->search_entry = gtk_entry_new ();
 	gtk_widget_show (view->priv->search_entry);
-	
-	g_signal_connect (view->priv->search_entry, "populate_popup",
+
+	g_signal_connect (view->priv->search_entry, "populate-popup",
 			  G_CALLBACK (search_entry_populate_popup),
 			  view);
+	g_signal_connect (view->priv->search_entry, "icon-release",
+			  G_CALLBACK (search_entry_icon_release),
+			  view);
 	g_signal_connect (view->priv->search_entry, "activate", 
 			  G_CALLBACK (search_entry_activate),
 			  view);
@@ -2120,6 +2159,19 @@ gedit_view_button_press_event (GtkWidget *widget, GdkEventButton *event)
 	return GTK_WIDGET_CLASS (gedit_view_parent_class)->button_press_event (widget, event);
 }
 
+static void
+gedit_view_realize (GtkWidget *widget)
+{
+	GeditView *view = GEDIT_VIEW (widget);
+
+	/* We only activate the extensions when the view is realized,
+	 * because most plugins will expect this behaviour, and we won't
+	 * change the buffer later anyway. */
+	peas_extension_set_call (view->priv->extensions, "activate");
+
+	GTK_WIDGET_CLASS (gedit_view_parent_class)->realize (widget);
+}
+
 static void 	
 search_highlight_updated_cb (GeditDocument *doc,
 			     GtkTextIter   *start,
